package javax.rmi.CORBA;

import java.io.SerializablePermission;
import java.net.MalformedURLException;
import java.rmi.server.RMIClassLoader;
import java.util.Properties;
import org.omg.CORBA.INITIALIZE;
import com.sun.corba.se.impl.orbutil.GetPropertyAction;
import org.omg.CORBA.ORB;
import java.rmi.NoSuchObjectException;
import java.rmi.Remote;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import java.rmi.RemoteException;
import org.omg.CORBA.SystemException;
import java.security.AccessController;
import java.security.PrivilegedAction;

public class Util
{
    private static final UtilDelegate utilDelegate;
    private static final String UtilClassKey = "javax.rmi.CORBA.UtilClass";
    private static final String ALLOW_CREATEVALUEHANDLER_PROP = "jdk.rmi.CORBA.allowCustomValueHandler";
    private static boolean allowCustomValueHandler;
    
    private static boolean readAllowCustomValueHandlerProperty() {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                return Boolean.valueOf(Boolean.getBoolean("jdk.rmi.CORBA.allowCustomValueHandler"));
            }
        });
    }
    
    private Util() {
    }
    
    public static RemoteException mapSystemException(SystemException ex) {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.mapSystemException(ex);
        }
        return null;
    }
    
    public static void writeAny(OutputStream outputStream, Object o) {
        if (Util.utilDelegate != null) {
            Util.utilDelegate.writeAny(outputStream, o);
        }
    }
    
    public static Object readAny(InputStream inputStream) {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.readAny(inputStream);
        }
        return null;
    }
    
    public static void writeRemoteObject(OutputStream outputStream, Object o) {
        if (Util.utilDelegate != null) {
            Util.utilDelegate.writeRemoteObject(outputStream, o);
        }
    }
    
    public static void writeAbstractObject(OutputStream outputStream, Object o) {
        if (Util.utilDelegate != null) {
            Util.utilDelegate.writeAbstractObject(outputStream, o);
        }
    }
    
    public static void registerTarget(Tie tie, Remote remote) {
        if (Util.utilDelegate != null) {
            Util.utilDelegate.registerTarget(tie, remote);
        }
    }
    
    public static void unexportObject(Remote remote) throws NoSuchObjectException {
        if (Util.utilDelegate != null) {
            Util.utilDelegate.unexportObject(remote);
        }
    }
    
    public static Tie getTie(Remote remote) {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.getTie(remote);
        }
        return null;
    }
    
    public static ValueHandler createValueHandler() {
        isCustomSerializationPermitted();
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.createValueHandler();
        }
        return null;
    }
    
    public static String getCodebase(Class clazz) {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.getCodebase(clazz);
        }
        return null;
    }
    
    public static Class loadClass(String s, String s2, ClassLoader classLoader) throws ClassNotFoundException {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.loadClass(s, s2, classLoader);
        }
        return null;
    }
    
    public static boolean isLocal(Stub stub) throws RemoteException {
        return Util.utilDelegate != null && Util.utilDelegate.isLocal(stub);
    }
    
    public static RemoteException wrapException(Throwable t) {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.wrapException(t);
        }
        return null;
    }
    
    public static Object[] copyObjects(Object[] array, ORB orb) throws RemoteException {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.copyObjects(array, orb);
        }
        return null;
    }
    
    public static Object copyObject(Object o, ORB orb) throws RemoteException {
        if (Util.utilDelegate != null) {
            return Util.utilDelegate.copyObject(o, orb);
        }
        return null;
    }
    
    private static Object createDelegate(String key) {
        String property = (String)AccessController.doPrivileged(new GetPropertyAction(key));
        if (property == null) {
            Properties orbPropertiesFile = getORBPropertiesFile();
            if (orbPropertiesFile != null) {
                property = orbPropertiesFile.getProperty(key);
            }
        }
        if (property == null) {
            return new com.sun.corba.se.impl.javax.rmi.CORBA.Util();
        }
        try {
            return loadDelegateClass(property).newInstance();
        }
        catch (ClassNotFoundException ex) {
            INITIALIZE o = new INITIALIZE("Cannot instantiate " + property);
            o.initCause(ex);
            throw o;
        }
        catch (Exception ex2) {
            INITIALIZE o2 = new INITIALIZE("Error while instantiating" + property);
            o2.initCause(ex2);
            throw o2;
        }
    }
    
    private static Class loadDelegateClass(String str) throws ClassNotFoundException {
        try {
            return Class.forName(str, false, Thread.currentThread().getContextClassLoader());
        }
        catch (ClassNotFoundException ex) {
            try {
                return RMIClassLoader.loadClass(str);
            }
            catch (MalformedURLException ex2) {
                throw new ClassNotFoundException("Could not load " + str + ": " + ex2.toString());
            }
        }
    }
    
    private static Properties getORBPropertiesFile() {
        return (Properties)AccessController.doPrivileged(new GetORBPropertiesFileAction());
    }
    
    private static void isCustomSerializationPermitted() {
        SecurityManager securityManager = System.getSecurityManager();
        if (!Util.allowCustomValueHandler && securityManager != null) {
            securityManager.checkPermission(new SerializablePermission("enableCustomValueHandler"));
        }
    }
    
    static {
        utilDelegate = (UtilDelegate)createDelegate("javax.rmi.CORBA.UtilClass");
        Util.allowCustomValueHandler = readAllowCustomValueHandlerProperty();
    }
}
